@@ -68,6 +68,128 @@ def spim(word, model,words_list1,words_list2):
             closest_index = i
             closest_distance = distance
     return words_list2[closest_index]
+def spimx3(word,model,words_list1,words_list2):
+    import re
+    import nltk
+    import random
+    from nltk.corpus import stopwords
+    from nltk.tokenize import word_tokenize
+    from nltk.stem import PorterStemmer
+
+    nltk.download('stopwords', quiet=True)
+    nltk.download('punkt', quiet=True)
+
+    def preprocess_text(text):
+        # Convert to lowercase
+
+        # Remove special characters and numbers
+        text = re.sub(r'[^a-z]+', ' ', text)
+        # Tokenize the text
+        words = word_tokenize(text)
+        # Remove stopwords
+        stop_words = set(stopwords.words('english'))
+        words = [word for word in words if word not in stop_words]
+        # Stem the words
+        stemmer = PorterStemmer()
+        words = [stemmer.stem(word) for word in words]
+        return words
+
+    def load_data(words_list1, words_list2):
+        input_data = words_list1
+        label_data = words_list2
+        return input_data, label_data
+
+    def get_similarity(word, words_list):
+        # Preprocess the input word
+        word = preprocess_text(word)
+        # Initialize a list to store the similarity scores
+        similarity_scores = []
+        # Iterate through each word in the words list
+        for w in words_list:
+            # Preprocess the word in the words list
+            w_processed = preprocess_text(w)
+            # Calculate the similarity score between the two words
+            if len(set(w_processed).union(set(word))) == 0:
+                score = random.randint(0, 20)
+            else:
+                score = len(set(word).intersection(w_processed)) / len(set(w_processed).union(set(word)))
+            similarity_scores.append(score)
+        # Return the index of the most similar word
+        return similarity_scores.index(max(similarity_scores))
+
+    # Load the data from the CSV file
+    input_data, label_data = load_data(words_list1, words_list2)
+
+    # Get the user input
+    input_word = word
+
+    # Find the index of the most similar word in the input data
+    index = get_similarity(input_word, input_data)
+
+    # Output the corresponding label from the label data
+    return label_data[index]
+
+
+def spimx2(word,model,words_list1,words_list2):
+    import re
+    import nltk
+    import random
+    from nltk.corpus import stopwords
+    from nltk.tokenize import word_tokenize
+    from nltk.stem import PorterStemmer
+
+    nltk.download('stopwords', quiet=True)
+    nltk.download('punkt', quiet=True)
+
+    def preprocess_text(text):
+        # Convert to lowercase
+        text = text.lower()
+        # Remove special characters and numbers
+        text = re.sub(r'[^a-z]+', ' ', text)
+        # Tokenize the text
+        words = word_tokenize(text)
+        # Remove stopwords
+        stop_words = set(stopwords.words('english'))
+        words = [word for word in words if word not in stop_words]
+        # Stem the words
+        stemmer = PorterStemmer()
+        words = [stemmer.stem(word) for word in words]
+        return words
+
+    def load_data(words_list1, words_list2):
+        input_data = words_list1
+        label_data = words_list2
+        return input_data, label_data
+
+    def get_similarity(word, words_list):
+        # Preprocess the input word
+        word = preprocess_text(word)
+        # Initialize a list to store the similarity scores
+        similarity_scores = []
+        # Iterate through each word in the words list
+        for w in words_list:
+            # Preprocess the word in the words list
+            w_processed = preprocess_text(w)
+            # Calculate the similarity score between the two words
+            if len(set(w_processed).union(set(word))) == 0:
+                score = random.randint(0, 20)
+            else:
+                score = len(set(word).intersection(w_processed)) / len(set(w_processed).union(set(word)))
+            similarity_scores.append(score)
+        # Return the index of the most similar word
+        return similarity_scores.index(max(similarity_scores))
+
+    # Load the data from the CSV file
+    input_data, label_data = load_data(words_list1, words_list2)
+
+    # Get the user input
+    input_word = word
+
+    # Find the index of the most similar word in the input data
+    index = get_similarity(input_word, input_data)
+
+    # Output the corresponding label from the label data
+    return label_data[index]
 
 
 def spimx(word,model,words_list1,words_list2):
@@ -120,7 +242,67 @@ def spimx(word,model,words_list1,words_list2):
 
              similarity_scores.append(score)
             # Return the index of the most similar word
-             return similarity_scores.index(max(similarity_scores))
+         return similarity_scores.index(max(similarity_scores))
+
+            # Load the data from the CSV file
+
+    input_data, label_data = load_data(words_list1, words_list2)
+        # Get the user input
+    input_word = word
+        # Find the index of the most similar word in the input data
+    index = get_similarity(input_word, input_data)
+        # Output the corresponding label from the label data
+    return label_data[index]
+
+def spimx4(word,model,words_list1,words_list2):
+    import re
+    import nltk
+    from nltk.corpus import stopwords
+    from nltk.tokenize import word_tokenize
+    from nltk.stem import PorterStemmer
+    nltk.download('stopwords',quiet=True)
+    nltk.download('punkt',quiet=True)
+
+    def preprocess_text(text):
+            # Convert to lowercase
+        text = text.lower()
+            # Remove special characters and numbers
+        text = re.sub(r'[^a-z]+', ' ', text)
+            # Tokenize the text
+        words = word_tokenize(text)
+            # Remove stopwords
+        stop_words = set(stopwords.words('english'))
+        words = [word for word in words if word not in stop_words]
+            # Stem the words
+        stemmer = PorterStemmer()
+        words = [stemmer.stem(word) for word in words]
+        return words
+
+    def load_data(words_list1, words_list2):
+        # Preprocess input data
+        input_data = [preprocess_text(text) for text in words_list1]
+        # Preprocess label data
+        label_data = [preprocess_text(text) for text in words_list2]
+        return input_data, label_data
+
+    def get_similarity(word, words_list):
+            # Preprocess the input word
+        word = preprocess_text(word)[0]
+            # Initialize a list to store the similarity scores
+        similarity_scores = []
+            # Iterate through each word in the words list
+        for w in words_list:
+                # Preprocess the word in the words list
+            w_processed = w
+                # Calculate the similarity score between the two words
+            if len(set(w_processed).union(w)) == 0:
+               score = random.randint(0, 20)
+            else:
+               score = len(set(word).intersection(w_processed)) / len(set(word).union(w_processed))
+
+            similarity_scores.append(score)
+            # Return the index of the most similar word
+        return similarity_scores.index(max(similarity_scores))
 
             # Load the data from the CSV file
 
@@ -555,6 +737,12 @@ def model_load(modeltype,input, model):
         return spim(input,model,words_list1,words_list2)
     elif modeltype == "spimx":
         return spimx(input, model, words_list1, words_list2)
+    elif modeltype == "spimx2":
+        return spimx2(input, model, words_list1, words_list2)
+    elif modeltype == "spimx3":
+        return spimx3(input, model, words_list1, words_list2)
+    elif modeltype == "spimx4":
+        return spimx4(input, model, words_list1, words_list2)
     elif modeltype == "rasv":
         return rasv(input,model,words_list1,words_list2)
     elif modeltype == "spimxr":
